#!/usr/bin/env python
# -*- coding: utf8 -*-
# PlumHound (phTaskzipper.py) - Create Zip of tasks
# https://github.com/PlumHound/PlumHound 
# License GNU GPL3

#Python Libraries
from datetime import date
from zipfile import ZipFile

#Plumhound Modules
from lib.phLoggy import Loggy as Loggy

def ZipTasks(verbose,Processed_Results_List, OutPathFile, Outpath):
    Loggy(verbose,900, "------ENTER: Task Zipper-----")
    Loggy(verbose,200, "Preparing ZipFile " + OutPathFile)
    with ZipFile(OutPathFile,"w") as NewZipFile:
        for entry in Processed_Results_List:
            filename=Outpath + entry[2]
            if entry[3] == "HTMLCSV":
                Loggy(verbose,200, "Zip-ADD: " + filename + ".csv Into " + OutPathFile)
                NewZipFile.write(filename + ".csv")
                Loggy(verbose,200, "Zip-ADD: " + filename + ".html Into " + OutPathFile)
                NewZipFile.write(filename + ".html")
            else:
                Loggy(verbose,200, "Zip-ADD: " + filename + " Into " + OutPathFile)
                NewZipFile.write(filename)

    Loggy(verbose,99, "Completed Reports Archive: " + OutPathFile)
    Loggy(verbose,900, "------EXIT: Task Zipper-----")
    return True





from py2neo import Graph, Node, Relationship

#######################################################
#
# Change the variable below to reflect your environment
#
#######################################################

neodb="bolt://localhost:7687"
neouser="neo4j"
neopass="password"

# End of config

def analyzepath(fneodb,fneouser,fneopass,startnode,endnode):
    # This fonction is used when a start node and a and node is specified.
    query="""MATCH p=shortestpath((n {name: $snode})-[*1..]->(m {name: $enode}))
        UNWIND relationships(p) as relToOmit
        OPTIONAL MATCH path = shortestPath((n)-[*1..]->(m))
        RETURN startNode(relToOmit).name as snode, endNode(relToOmit).name as enode, type(relToOmit) as rel, id(relToOmit) as relId"""
    graph = Graph(fneodb,user=fneouser,password=fneopass)
    result=graph.run(query,snode=startnode,enode=endnode) 
    if result:
        while result.forward():
            print("Removing the relationship", result.current["rel"] ,"between", result.current["snode"] ,"and", result.current["enode"] ,"breaks the path!")
    else:
        print("There is no path between startnode and endnode")


def getpaths(fneodb,fneouser,fneopass,startnode,endnode):
    # This function will get the shortest paths from StarNode and EndNode
    # First we initiate our graph
    graph = Graph(fneodb,user=fneouser,password=fneopass)
    # Test if we received a label or 2 nodes and build the query accordingly
    if startnode == "AZUser" or startnode == "AZGroup" or startnode == "AZDevice" or startnode == "AZServicePrincipal" or startnode == "AZApp":
        query='MATCH p=ShortestPath((n:%s)-[*1..]->(m:AZRole)) WHERE m.name STARTS WITH "%s@" AND n <> m RETURN p' % (startnode, endnode)
    else:
        query='MATCH p=ShortestPath((n {name: "%s"})-[*1..]->(m {name: "%s"})) WHERE n <> m RETURN p' % (startnode,endnode)
    paths=graph.run(query)
    # Extract the starting & ending node of each paths and send them to be analyzed
    path=paths.evaluate()
    if path == None:
        print ("---------------------------------------------------------------------")
        print("There is no path between", startnode,"and", endnode)
        print ("---------------------------------------------------------------------")
    while path != None:
        snode=path.start_node["name"]
        enode=path.end_node["name"]
        print ("---------------------------------------------------------------------")
        print ("Analyzing paths between", snode, "and", enode)
        print ("---------------------------------------------------------------------")
        analyzepath(fneodb,fneouser,fneopass,snode,enode)
        path=paths.evaluate()
        pass

# Unimplemented

# def find_busiest_path(fneodb,fneouser,fneopass,param,number):
#     # This function counts how many principals have the same path
#     # The goal is to find the path(s) which give the most users a path and focus on it to remediate
#     graph = Graph(fneodb,user=fneouser,password=fneopass)
#     if param.lower() == "all":
#         query_shortpath="""MATCH p=allShortestPaths((n)-[*1..]->(m:AZRole)) where m.name starts with "GLOBAL ADMIN" and n<>m RETURN p"""
#     else: 
#         query_shortpath="""MATCH p=ShortestPath((n)-[*1..]->(m:AZRole)) where m.name starts with "GLOBAL ADMIN" and n<>m RETURN p"""
#     paths=graph.run(query_shortpath)
#     path=paths.evaluate()
#     if path == None:
#         print ("---------------------------------------------------------------------")
#         print("There is no ShortestPath")
#         print ("---------------------------------------------------------------------")
#     x=0
#     ucount=[]
#     while path != None:
#         snode=path.start_node["name"]
#         query_count_user="""match p=shortestpath((s)-[*1..]->(d:AZRole))
# where s.name = "%s" and d.name starts with "GLOBAL ADMIN" and s<>d
# unwind (NODES(p)) as pathNodes
# match (pathNodes) where labels(pathNodes) = ["AZBase", "AZRole"]
# with pathNodes as grp
# match (grp)<-[:MemberOf*1..]-(u:AZUser)
# return count(distinct(u))
#         """ % snode
#         usercount=graph.run(query_count_user)
#         nb_user=usercount.evaluate()
#         result=[nb_user, snode]
#         ucount.append(result)
#         x=x+1
#         path=paths.evaluate()
#     ucount.sort(reverse=True)
#     i=0
#     nb_paths=int(number)
#     while i<nb_paths:
#         try:
#             print(ucount[i])
#             i=i+1
#         except Exception as e: # If there are less results than specified.
#             i=i+1
#     return (ucount)


def main():
    # Nothing here as this code should be used with Plumhound : https://github.com/PlumHound/PlumHound
    x=0

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf8 -*-
# PlumHound (phCLIManagement.py) - Management of Command Line Arguments
# https://github.com/PlumHound/PlumHound 
# License GNU GPL3

# Import Python Modules
import argcomplete
import argparse

def SetupArguments(ph_version):
    parser = argparse.ArgumentParser(description="BloodHound Wrapper for Blue/Purple Teams "+str(ph_version), add_help=True, epilog="For more information see https://github.com/plumhound")
    argcomplete.autocomplete(parser)
    
    pgroupc = parser.add_argument_group('DATABASE')
    pgroupc.add_argument("-s", "--server", type=str, help="Neo4J Server", default="bolt://localhost:7687")
    pgroupc.add_argument("-u", "--username", default="neo4j", type=str, help="Neo4J Database Useranme")
    pgroupc.add_argument("-p", "--password", default="neo4jneo4j", type=str, help="Neo4J Database Password")
    pgroupc.add_argument("--UseEnc", default=False, dest="UseEnc", help="Use encryption when connecting.", action='store_true')
    pgroupc.add_argument("--timeout", type=int, default="300", dest="timeout", help="Cypher Query Timeout **NOT FULLY IMPLEMENTED**")

    pgroupx = parser.add_mutually_exclusive_group(required="True")
    pgroupx.add_argument("--easy", help="Test Database Connection, Returns Domain Users to stdout", action='store_true')
    pgroupx.add_argument("-x", "--TaskFile", dest="TaskFile", type=str, help="Specify a PlumHound TaskList File")
    pgroupx.add_argument("-q", "--QuerySingle", dest="QuerySingle", type=str, help="Specify a Single Cypher Query")
    pgroupx.add_argument("-bp", "--BusiestPath", dest="BusiestPath", nargs='+', default=False, type=str, help="Find the X Shortest Paths that give the most users a path to Domain Admins. Need to specified [short|all] for shortestpath and the number of results. Ex: PlumHound -cu all 3")
    pgroupx.add_argument("-ap", "--AnalyzePath", dest="AnalyzePath", nargs='+', default=False, type=str, help="Analyze 'Attack Paths' between two nodes and find which path needs to be remediated to brake the path.")
    pgroupx.add_argument("-az", "--AzPath", dest="AzPath", nargs='+', default=False, type=str, help="Analyze 'Attack Paths' between two nodes in Azure. Specify starting and ending nodes or use the built-in options to find paths to Global Administrator: [AZUser|AZGroup|AZApp|AZServicePrincipal|AZDevice]. ALL can be used to run all the built-in options at once.")

    pgroupo = parser.add_argument_group('OUTPUT', "Output Options (For single cypher queries only. --These options are ignored when -x or --easy is specified.")
    pgroupo.add_argument("-t", "--title", dest="title", default="Adhoc Query", type=str, help="Report Title for Single Query [HTML,CSV,Latex]")
    pgroupo.add_argument("--of", "--OutFile", dest="OutFile", default="PlumHoundReport", type=str, help="Specify a Single Cypher Query")
    pgroupo.add_argument("--op", "--OutPath", dest="path", default="reports//", type=str, help="Specify an Output Path for Reports")
    pgroupo.add_argument("--ox", "--OutFormat", dest="OutFormat", default="STDOUT", type=str, help="Specify the type of output.", choices=['stdout', 'HTML', 'CSV', 'HTMLCSV'])

    pgrouph = parser.add_argument_group('HTML', "Options for HTML Output (For single queries or TaskLists")
    pgrouph.add_argument("--HTMLHeader", dest="HTMLHeader", type=str, default="template//head.html", help="HTML Header (file) of Report")
    pgrouph.add_argument("--HTMLFooter", dest="HTMLFooter", type=str, default="template//tail.html", help="HTML Footer (file) of Report")
    pgrouph.add_argument("--HTMLCSS", dest="HTMLCSS", type=str, default="template//html.css", help="Specify a CSS template for HTML Output")

    pgroupv = parser.add_argument_group('VERBOSE', "Set verbosity")
    pgroupv.add_argument("-v", "--verbose", type=int, default="100", help="Verbosity 0-1000, 0 = quiet, default=100, info=150, verbose=1000")

    args = parser.parse_args()

    return args




#!/usr/bin/env python
# -*- coding: utf8 -*-
# PlumHound (phCheckPython.py) - Check for appropriate version of python
# https://github.com/PlumHound/PlumHound 
# License GNU GPL3

import sys
from lib.phLoggy import Loggy as Loggy

def CheckPython2():
    if sys.version_info < (3, 0, 0):
        print(__file__ + ' requires Python 3, while Python ' + str(sys.version[0] + ' was detected. Terminating. '))
        sys.exit(1)

def CheckPython3(phArgs):
    if sys.hexversion > 0x3090000:
        Loggy(phArgs.verbose,200, "Python Hex Version Identified:"+str(sys.hexversion))




from py2neo import Graph, Node, Relationship

#######################################################
#
# Change the variable below to reflect your environment
#
#######################################################

neodb="bolt://localhost:7687"
neouser="neo4j"
neopass="neo4jj"

# End of config

def analyzepath(fneodb,fneouser,fneopass,startnode,endnode):
    # This fonction is used when a start node and a and node is specified.
    query="""MATCH p=shortestpath((n {name: $snode})-[*1..]->(m {name: $enode}))
        UNWIND relationships(p) as relToOmit
        OPTIONAL MATCH path = shortestPath((n)-[*1..]->(m))
        WHERE none(rel in relationships(path) WHERE rel = relToOmit)
        WITH path, relToOmit
        WHERE path IS NULL
        RETURN startNode(relToOmit).name as snode, endNode(relToOmit).name as enode, type(relToOmit) as rel, id(relToOmit) as relId"""
    graph = Graph(fneodb,user=fneouser,password=fneopass)
    result=graph.run(query,snode=startnode,enode=endnode) 
    if result:
        while result.forward():
            print("Removing the relationship", result.current["rel"] ,"between", result.current["snode"] ,"and", result.current["enode"] ,"breaks the path!")
    else:
        print("There is no path between startnode and endnode")


def getpaths(fneodb,fneouser,fneopass,startnode,endnode):
    # This function will get the shortest paths from StarNode and EndNode
    # First we initiate our graph
    graph = Graph(fneodb,user=fneouser,password=fneopass)
    # Test if we received a label or 2 nodes and build the query accordingly
    if startnode == "User" or startnode == "Group" or startnode == "Computer":
        nodetype=startnode
        query='MATCH p=ShortestPath((n:%s)-[*1..]->(m:Group)) WHERE m.name STARTS WITH "DOMAIN ADMINS@" AND n <> m RETURN p' % (nodetype)
    else:
        query='MATCH p=ShortestPath((n {name: "%s"})-[*1..]->(m {name: "%s"})) WHERE n <> m RETURN p' % (startnode,endnode)
    paths=graph.run(query)
    # Extract the starting & ending node of each paths and send them to be analyzed
    path=paths.evaluate()
    if path == None:
        print ("---------------------------------------------------------------------")
        print("There is no path between", startnode,"and", endnode)
        print ("---------------------------------------------------------------------")
    while path != None:
        snode=path.start_node["name"]
        enode=path.end_node["name"]
        print ("---------------------------------------------------------------------")
        print ("Analyzing paths between", snode, "and", enode)
        print ("---------------------------------------------------------------------")
        analyzepath(fneodb,fneouser,fneopass,snode,enode)
        path=paths.evaluate()
        pass


def find_busiest_path(fneodb,fneouser,fneopass,param,number):
    # This function counts how many principals have the same path
    # The goal is to find the path(s) which give the most users a path and focus on it to remediate
    graph = Graph(fneodb,user=fneouser,password=fneopass)
    #ucount[0] = "a"
    if param.lower() == "all":
        query_shortpath="""MATCH p=allShortestPaths((n)-[*1..]->(m:Group)) where m.name starts with "DOMAIN ADMINS" and n<>m RETURN p"""
    else: 
        query_shortpath="""MATCH p=ShortestPath((n)-[*1..]->(m:Group)) where m.name starts with "DOMAIN ADMINS" and n<>m RETURN p"""
    paths=graph.run(query_shortpath)
    path=paths.evaluate()
    if path == None:
        print ("---------------------------------------------------------------------")
        print("There is no ShortestPath")
        print ("---------------------------------------------------------------------")
    x=0
    ucount=[]
    while path != None:
        snode=path.start_node["name"]
        query_count_user="""match p=shortestpath((s)-[*1..]->(d:Group))
where s.name = "%s" and d.name starts with "DOMAIN ADMINS" and s<>d
unwind (NODES(p)) as pathNodes
match (pathNodes) where labels(pathNodes) = ["Base", "Group"]
with pathNodes as grp
match (grp)<-[:MemberOf*1..]-(u:User)
return count(distinct(u))
        """ % snode
        usercount=graph.run(query_count_user)
        nb_user=usercount.evaluate()
        result=[nb_user, snode]
        ucount.append(result)
        x=x+1
        path=paths.evaluate()
    ucount.sort(reverse=True)
    i=0
    nb_paths=int(number)
    while i<nb_paths:
        print(ucount[i])
        i=i+1
    return (ucount)


def main():
    # Nothing here as this code should be used with Plumhound : https://github.com/PlumHound/PlumHound
    x=0

if __name__ == "__main__":
    main()
